import serial
import time
import random
from psychopy import core, visual, event
from pylsl import StreamInfo, StreamOutlet

from psychopy.hardware import keyboard

defaultKeyboard = keyboard.Keyboard()

Ntrials = 30
refresh_rate = 60 #Set the refresh rate of the screen

MI_trials = [1.0, 2.0, 3.0]

n_trials_per_class = int(Ntrials/3)

MI_trials = MI_trials * n_trials_per_class
random.shuffle(MI_trials)

# Setting up the display (rename the monitor if needed)
win = visual.Window(size=(1920,1080), fullscr=True, allowGUI=False, allowStencil=False, monitor='display1', color=[-1,-1,-1], colorSpace='rgb', blendMode='avg', useFBO=False, waitBlanking = True, units="pix")

# Creating the LSL stream:
info = StreamInfo(name='TriggerStream', type='Markers', channel_count=1, channel_format='int32', source_id='BP_MI_Marker_stream_001') #Initialize LSL stream for markers
outlet = StreamOutlet(info)  # Broadcast the stream.

# Just an example of some basic shapes
verts_right = [(0,30),(200,30),(200, 50), (300, 0), (200, -50),(200,-30),(0,-30)]
right = visual.ShapeStim(win, fillColor='green', vertices=verts_right, lineColor='green', opacity=1)

verts_left = [(0,30),(-200,30),(-200, 50), (-300, 0), (-200, -50),(-200,-30),(0,-30)]
left = visual.ShapeStim(win, fillColor='green', vertices=verts_left, lineColor='green', opacity=1)

verts_down = [(30,0),(30,-200),(50, -200), (0, -300), (-50, -200),(-30,-200),(-30,0)]
down = visual.ShapeStim(win, fillColor='green', vertices=verts_down, lineColor='green', opacity=1)

cross_line1 = visual.Line(win=win, units="pix", lineColor=[1, 1, 1], lineWidth=5)
cross_line1.start = [-200, 0]
cross_line1.end = [+200, 0]

cross_line2 = visual.Line(win=win, units="pix", lineColor=[1, 1, 1], lineWidth=5)
cross_line2.start = [0, -200]
cross_line2.end = [0, +200]

core.wait(10)

# Pushing different markers to the LSL stream: 
# 1 for the appearing cross or the begin of the trial
# 2 for the arrow pointing to the left
# 3 for the arrow pointing to the right
# 4 for the arrow pointing downward
# 8 for the disappearing cross or the end of the MI trial
for MI_trial in MI_trials:
    if defaultKeyboard.getKeys(keyList=['escape']): #stop the experiment with Escape
        core.quit()
    if MI_trial == 1:
        for iframe in range (round(2*refresh_rate)): #0-2s: only cross
            cross_line1.draw()
            cross_line2.draw()
            win.flip()
            if iframe==0:
                outlet.push_sample(x=[1])
        for iframe in range (round(1.25*refresh_rate)): #2-3.25s: cue
            cross_line1.draw()
            cross_line2.draw()
            left.draw()
            win.flip()
            if iframe==0:
                outlet.push_sample(x=[2])
        for iframe in range (round(4.75*refresh_rate)): #3.25-8s: only cross
            cross_line1.draw()
            cross_line2.draw()
            win.flip()
        outlet.push_sample(x=[8]) #end of trial
    if MI_trial == 2:
        for iframe in range (round(2*refresh_rate)): #0-2s: only cross
            cross_line1.draw()
            cross_line2.draw()
            win.flip()
            if iframe==0:
                outlet.push_sample(x=[1])
        for iframe in range (round(1.25*refresh_rate)): #2-3.25s: cue
            cross_line1.draw()
            cross_line2.draw()
            right.draw()
            win.flip()
            if iframe==0:
                outlet.push_sample(x=[3])
        for iframe in range (round(4.75*refresh_rate)): #3.25-8s: only cross
            cross_line1.draw()
            cross_line2.draw()
            win.flip()
        outlet.push_sample(x=[8]) #end of trial
    if MI_trial == 3:
        for iframe in range (round(2*refresh_rate)): #0-2s: only cross
            cross_line1.draw()
            cross_line2.draw()
            win.flip()
            if iframe==0:
                outlet.push_sample(x=[1])
        for iframe in range (round(1.25*refresh_rate)): #2-3.25s: cue
            cross_line1.draw()
            cross_line2.draw()
            down.draw()
            win.flip()
            if iframe==0:
                outlet.push_sample(x=[4])
        for iframe in range (round(4.75*refresh_rate)): #3.25-8s: only cross
            cross_line1.draw()
            cross_line2.draw()
            win.flip()
        outlet.push_sample(x=[8]) #end of trial
    win.flip()
    core.wait(random.randint(500,2500)/1000) #random inter-trial interval
            

win.close()